<?php
namespace PME\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;

class PratoController extends \PME\Services\TeknisaCrud {
    protected $crudService;
	protected $parameterBag;
	protected $connection;
	protected $environment;
	protected $pratoService;
	protected $dataSourceManager;
	protected $utilities;
	protected $entityManager;
	

    public function __construct($crudService, $parameterBag, $connection, $environment, $pratoService, $dataSourceManager, $utilities, $entityManager) {
        parent::__construct($crudService, $parameterBag);
        $this->connection        = $connection;
        $this->environment       = $environment;
        $this->pratoService      = $pratoService;
		$this->dataSourceManager = $dataSourceManager;
		$this->utilities         = $utilities;
		$this->entityManager     = $entityManager;
	}
	
	private function callSave($dataSourceName, $rowsToSave){
	    $dataSet = new DataSet($dataSourceName, $rowsToSave);
	    $request = new DTO\Request\DataSet($dataSet, "", "", "");  
	    $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
	    $this->crudService->save($dataSourceName, $rowsToSave);
	}
    
    public function retornaParametrosAdicionarPrato(DTO\Request\Row $request, DTO\Response $response){
    	$row = $request->getRow();
    	$this->validationDependingGroup($request, $response);
        
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('paramgeral', array());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $response->addDataSet($dataSet);
    }

	public function searchNewCode(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$nrorg = $this->utilities->getNrOrg();
		$pratoPai = $parameters['pratoPai'];
		$filAux = $parameters['filAux'];
		$levelPlate = $parameters['levelPlate'];
		$filial = $parameters['filial'];

		$lastChildren = null;
		$maxCodePlate = 0;
		$padRightParentPlate = str_pad($pratoPai, 10, "X", STR_PAD_RIGHT);
		$children = $this->returnChildren($pratoPai, $filAux, $filial, $nrorg);
		if (count($children) == 0) $children = $this->returnLevelComplete($nrorg); //caso não tenha nenhum registro selecionado
		sort($children);
		$maskPlate = $this->getCdPictPrat();
	
		if ($children) {
			$lastPosition = end($children);
			$lastChildren = $lastPosition['CDPRATO'];
		}
		
		if ($lastChildren === null) {
			$formatCode = $this->getFormatPlate($padRightParentPlate, $maskPlate);
			$matches = array();
			preg_match("/X+/", $formatCode, $matches);
			$sizeToComplete = strlen($matches[0]);
			$maxCodePlate = str_pad($pratoPai, $sizeToComplete+strlen($pratoPai), "0", STR_PAD_RIGHT);
		} else {
			$formatCodeLastPlate = $this->getFormatPlate($lastChildren, $maskPlate);
			$arrayCode = explode(".", $formatCodeLastPlate);
			$lastPart = $arrayCode[count($arrayCode)-1];
			if(preg_match("/^9+$/", $lastPart) == 1){
                $maxCodePlate = $lastChildren; 
            } else {
                $maxCodePlate = $lastChildren + 1;
            }
		}
		$response->addDataSet(new DataSet ('CODIGO', array($maxCodePlate)));
	}

	public function returnChildren($pratoPai, $filAux, $filial, $nrorg) {
		$sql = "SELECT CDFILAUXPRAT, CDPRATO, CDFILIAL, NMPRATO, CDNVPRATO FROM PRATO WHERE CDPRATONIVANTE = :P_CDPRATONIVANTE AND NRORG = :P_NRORG AND CDFILAUXPRAT IN (:P_CDFILAUXPRAT, :P_CDFILIAL)";

		$params = array(
			"P_CDPRATONIVANTE" => $pratoPai,
			"P_NRORG"          => $nrorg,
			"P_CDFILAUXPRAT"   => $filAux,
			"P_CDFILIAL"       => $filial
		);
		return $this->entityManager->getConnection()->fetchAll($sql, $params);
	}
	
	public function returnLevelComplete($nrorg) {
		$sql = "SELECT CDFILAUXPRAT, CDPRATO, CDFILIAL, NMPRATO, CDNVPRATO FROM PRATO WHERE CDNVPRATO = '1' AND NRORG = :P_NRORG ORDER BY CDPRATO";

		$params = array(
			
			"P_NRORG" => $nrorg
		);
		return $this->entityManager->getConnection()->fetchAll($sql, $params);
	}

	public function getCdPictPrat(){
        $sql = "SELECT CDPICTPRAT FROM PARAMGERAL";
        $connection =  $this->entityManager->getConnection();
        $stmt = $connection->prepare($sql);
        $stmt->execute();
        return $stmt->fetch()['CDPICTPRAT'];
    }

	public function getFormatPlate($code, $mask){
        $regex = '';
        $toReplace = '';
        preg_match_all("/[^\.]+/", $mask, $results);
        $results = $results[0];
        for($i = 0; $i < sizeof($results); $i++){
            $sizeMatch = strlen($results[$i]);
            $regex .= $i === 0 ? "(.{{$sizeMatch}})" : "(.{0,$sizeMatch})";
            $toReplace .= $i !== 0 ? '.' : $toReplace;
            $toReplace .= "$".($i+1);
        }
        $formatCode = preg_replace("/".$regex."/", $toReplace, $code);
        $cleanCode = preg_replace("/(\.+)$/","",$formatCode);
        return $cleanCode;
    }

	public function returnPlate(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$plate = $parameters['CDPRATO'];
		$nrorg = $this->utilities->getNrOrg();

		$sql = "SELECT * FROM PRATO WHERE CDFILAUXPRAT IN (:CDFILAUXPRAT, 'P') AND CDPRATO = :CDPRATO AND NRORG = :NRORG ORDER BY CDPRATO DESC";
		$params = array(
			"CDFILAUXPRAT" => 'P',
			"CDPRATO"      => $plate,
			"NRORG"        => $nrorg
		);

		$result = $this->entityManager->getConnection()->fetchAll($sql, $params);

		$conditions = array(
			array(
				'columnName' => 'NRORG',
				'operator' => '=',
				'value' => $nrorg
			),
			array(
				'columnName' => 'CDPRATO',
				'operator' => '=',
				'value' => $plate
			)
		);
		$dataSet = $this->crudService->getFilteredDataset("prato_add", $conditions);

		if ($result) $response->addDataSet(new DataSet('PRATO', array('CDPRATO' => $result[0]['CDPRATO'], 'CDFILAUXPRAT' => $result[0]['CDFILAUXPRAT'], 'PRATOUNIDADE' => !empty($dataSet->getRows()))));
		else $response->addDataSet(new DataSet('PRATO', array('CDPRATO' => false, 'PRATOUNIDADE' => !empty($dataSet->getRows()))));
	}

	public function updatePackaging(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$rows = $parameters['update'];
		$nrorg = $this->utilities->getNrOrg();
		$this->connection->beginTransaction();
		$sql = "UPDATE PRATOCONTAINER SET IDPRATCONTPRI = :IDPRATCONTPRI WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND CDCONTAINER = :CDCONTAINER AND QTPRATCONTAINER = :QTPRATCONTAINER AND NRORG = :NRORG";

		$params = array(
			"CDFILAUXPRAT"    => $rows['CDFILAUXPRAT'],
			"CDPRATO"         => $rows['CDPRATO'],
			"CDCONTAINER"     => $rows['CDCONTAINER'],
			"QTPRATCONTAINER" => $rows['QTPRATCONTAINER'],
			"IDPRATCONTPRI"   => 'N',
			"NRORG"           => $nrorg
		);
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
		$this->connection->commit();
	}
	
	public function savePlate(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$row = $parameters['rowForm'];
		$nrorg = $this->utilities->getNrOrg();
		$this->connection->beginTransaction();

		$sql = "INSERT INTO PRATO (CDFILIAL, CDPRATO, NMPRATO, CDFILAUXPRAT, CDNVPRATO, SGUNIDADE, VRARREDTRANS, QTTEMPOPRAT, IDFREQUENCIA, IDATIVO, DTINATIVAUTO, CDPRODPRODUCAO, NRORG) VALUES(:CDFILIAL, :CDPRATO, :NMPRATO, :CDFILAUXPRAT, :CDNVPRATO, :SGUNIDADE, :VRARREDTRANS, :QTTEMPOPRAT, :IDFREQUENCIA, :IDATIVO, :DTINATIVAUTO, :CDPRODPRODUCAO, :NRORG)";
		$params = array(
			"CDFILIAL"       => $row["CDFILIAL"],
			"CDPRATO"        => $row["CDPRATO"],
			"NMPRATO"        => $row["NMPRATO"],
			"CDFILAUXPRAT"   => $row["CDFILAUXPRAT"],
			"CDNVPRATO"      => $row["CDNVPRATO"],
			"SGUNIDADE"      => $row["DSUNIDADE"],
			"VRARREDTRANS"   => $row["VRARREDTRANS"],
			"QTTEMPOPRAT"    => $row["QTTEMPOPRAT"],
			"IDFREQUENCIA"   => $row["IDFREQUENCIA"],
			"IDATIVO"        => $row["IDATIVO"],
			"DTINATIVAUTO"   => $row["DTINATIVAUTO"],
			"CDPRODPRODUCAO" => $row["NMFORMAPREPARO"],
			"NRORG"          => $nrorg
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
		$this->connection->commit();
	}
	
	public function updatePlate(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$rows = $parameters['editedRows'];
		$nrorg = $this->utilities->getNrOrg();
		$this->connection->beginTransaction();

		$sql = "UPDATE PRATO SET NMPRATO = :NMPRATO WHERE CDNVPRATO = :CDNVPRATO AND CDFILIAL = :CDFILIAL AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND NRORG = :NRORG";
		forEach($rows as $row) {
			$params = array(
				"CDFILIAL"       => $row["CDFILIAL"],
				"CDPRATO"        => $row["CDPRATO"],
				"NMPRATO"        => $row["NMPRATO"],
				"CDFILAUXPRAT"   => $row["CDFILAUXPRAT"],
				"CDNVPRATO"      => $row["CDNVPRATO"],
				"NRORG"          => $nrorg
			);
			
			$stmt = $this->connection->prepare($sql);
			$stmt->execute($params);
			$this->connection->commit();
		}
	}
	
	public function deletePlate(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$rows = $parameters['editedRows'];
		$nrorg = $this->utilities->getNrOrg();
		$this->connection->beginTransaction();
		$sql = "DELETE FROM PRATO WHERE NMPRATO = :NMPRATO AND CDNVPRATO = :CDNVPRATO AND CDFILIAL = :CDFILIAL AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND NRORG = :NRORG";
		forEach($rows as $row) {
			$params = array(
				"CDFILIAL"       => $row["CDFILIAL"],
				"CDPRATO"        => $row["CDPRATO"],
				"NMPRATO"        => $row["NMPRATO"],
				"CDFILAUXPRAT"   => $row["CDFILAUXPRAT"],
				"CDNVPRATO"      => $row["CDNVPRATO"],
				"NRORG"          => $nrorg
			);
			
			$stmt = $this->connection->prepare($sql);
			$stmt->execute($params);
			$this->connection->commit();
		}
	}
	
	public function resultDadosPlate(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$rows = $parameters['rowDadosPlate'];
		$nrorg = $this->utilities->getNrOrg();

		$sql = "SELECT P.CDFILIAL, P.IDATIVO, P.CDNVPRATO, P.CDPRATONIVANTE, P.CDPRATO, P.CDFILAUXPRAT, P.NMPRATO, CASE P.CDFILAUXPRAT WHEN 'P' THEN 'P' ELSE 'L' END AS IDTIPO, P.SGUNIDADE, UN.DSUNIDADE, P.NRFORMAPREPARO, FP.NMFORMAPREPARO, P.NRORG FROM PRATO          P LEFT JOIN UNIDMEDIDA     UN ON UN.SGUNIDADE = P.SGUNIDADE LEFT JOIN FORMAPREPARO   FP ON P.NRFORMAPREPARO = FP.NRFORMAPREPARO WHERE P.CDPRATO = :CDPRATO AND P.CDFILAUXPRAT = :CDFILAUXPRAT AND P.NMPRATO = :NMPRATO AND P.NRORG = :NRORG AND P.CDFILIAL = :CDFILIAL";
		
		$params = array(
			"CDFILIAL"       => $rows["CDFILIAL"],
			"CDPRATO"        => $rows["CDPRATO"],
			"NMPRATO"        => $rows["NMPRATO"],
			"CDFILAUXPRAT"   => $rows["CDFILAUXPRAT"],
			"NRORG"          => $nrorg
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    	$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}
	
	public function returnPercentSafra(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();

		$sql = "SELECT NRCORSINSAFR FROM SINASAFRA WHERE VRLIMINFSAFR <  :P_PERCSAFRA AND VRLIMSUPSAFR >= :P_PERCSAFRA OR :P_PERCSAFRA =  0 AND VRLIMINFSAFR =  :P_PERCSAFRA";
		
		$params = array(
			"P_PERCSAFRA" => $parameters['percentualSafra']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    	$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}

	public function validationDependingGroup($request, $response){
		$row = $request->getRow();
		$cdGrupOper = null;
		$perfAcess = null;
		$permission = true;
		
		$perfAcess = $this->getPerfilAcesso($row['ALIAS_NAME']);
		if(!$perfAcess){
			if($row['COLUMN_NAME']){
				$cdGrupOper = $this->getGrupOperParamGeral($row['COLUMN_NAME']);
			 } else{
				$cdGrupOper = $this->getGrupOperAutgrupop($row['ALIAS_NAME']);
			 }
		}
	
		if(!$cdGrupOper && !$perfAcess){
			$str = 'NOGROUP';
			$response->addDataSet(new DataSet ('permission', array('result' => $str)));
		}else{
			if($perfAcess){
				$result = $this->validPerfilAcesso($perfAcess, $this->environment->getCdOperador());
			}else{
				$result = $this->validGrupOper($cdGrupOper, $this->environment->getCdOperador());
			}
			$str = !empty($result) ? "VALID" : 'NOPERMISSION';
			$response->addDataSet(new DataSet ('permission', array('result' => $str)));
		}
	}

	public function getPerfilAcesso($idautperfacesso){
        $dataSourceName = 'autperfilacesso';
		$conditions = array(
			array(
				'columnName' => 'IDAUTPERFACESSO',
				'operator' => 'LIKE',
				'value' => $idautperfacesso
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		
		if(!empty($result)){
		    return $result[0]['NRPERFILACESSO'];
		}else{
		    return null;
		}
    }

	public function validGrupOper($cdGrupOper, $cdOperador){
        $dataSourceName = 'opergrupop';
		$conditions = array(
			array(
				'columnName' => 'CDGRUPOPER',
				'operator' => '=',
				'value' => $cdGrupOper
			),
			array(
				'columnName' => 'CDOPERADOR',
				'operator' => '=',
				'value' => $cdOperador
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		return $result;
    }

	public function validPerfilAcesso($nrperfilacesso, $cdOperador){
        $dataSourceName = 'perfAcessParcOperador';
		$conditions = array(
			array(
				'columnName' => 'NRPERFILACESSO',
				'operator' => '=',
				'value' => $nrperfilacesso
			),
			array(
				'columnName' => 'CDOPERADOR',
				'operator' => '=',
				'value' => $cdOperador
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		return $result;
    }

	public function getGrupOperParamGeral($columnName){
        $dataSourceName = 'paramgeral';
		$conditions = array();
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		
		if(!empty($result)){
		    return $result[0][$columnName];
		}else{
		    return null;
		}
    }

	public function getGrupOperAutgrupop($idautgrupop){
        $dataSourceName = 'autgrupop';
		$conditions = array(
			array(
				'columnName' => 'IDAUTGRUPOP',
				'operator' => 'LIKE',
				'value' => $idautgrupop
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		
		if(!empty($result)){
		    return $result[0]['CDGRUPOPER'];
		}else{
		    return null;
		}
    }
}
