create or replace PROCEDURE RETIRADA_COZINHA_CENTRAL(
   P_CDFILIAL     IN CARDIAPME.CDFILIAL%TYPE,
   P_CDSERVICO    IN SERVICO.CDSERVFILI%TYPE,
   P_DTRETIRADA   IN RETIRADA.DTRETIRADA%TYPE,
   P_DTCARDAPIO   IN CARDADIA.DTCARDAPIO%TYPE,
   P_CDOPERADOR   IN CARDADIA.CDOPERADOR%TYPE,
   P_CDFILIPROC   IN CARDIAPME.CDFILIAL%TYPE,
   P_DSLANCAMENTO IN LANCTOESTOQ.DSLANCESTQ%TYPE
) AS
   V_CDTIPORETI   VARCHAR2(2);
   V_NRSEQUITRETI NUMBER;
   V_QTCONSPLAN   NUMBER;
   V_QTDEESTO     NUMBER;
   V_VALRESTO     NUMBER;
   V_VALRBRUT     NUMBER;
   V_PERCENT      NUMBER;
   V_NRCARDAPIO   VARCHAR2(20);
   V_COUNT        NUMBER;
   V_CDFILIAL     VARCHAR2(4);
   V_NRLANCTOEST  VARCHAR2(20);
   V_NRRETIRADA   VARCHAR2(20);
   V_QTCONSUMIDA  NUMBER;
   V_CDALMOXARIFE VARCHAR2(6);
   V_CDLOCALESTOQ VARCHAR2(6);
   V_IDUTILLOTE   VARCHAR2(1);
   V_LOTEESTQ     VARCHAR2(20);
   V_SUBLOTE      VARCHAR2(4);
BEGIN

   /*Tipo de Retirada default*/
   BEGIN
      SELECT CDTPRETIPADR
         INTO V_CDTIPORETI
      FROM PARAMPLANEJA
      WHERE CDFILIAL = P_CDFILIAL;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CDTIPORETI := NULL;
   END;
   
   SELECT IDUTILLOTE
     INTO V_IDUTILLOTE
   FROM PARAMFILIAL
   WHERE CDFILIAL = P_CDFILIAL; 


   V_NRSEQUITRETI := 0;

   /*Verifica se existe cardpio para a data de retirada*/
   BEGIN
      SELECT NRCARDAPIO
         INTO V_NRCARDAPIO
      FROM CARDADIA
      WHERE (CDFILIAL   = P_CDFILIAL)
        AND (CDSERVFILI = P_CDSERVICO)
        AND (DTCARDAPIO = P_DTCARDAPIO);
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_NRCARDAPIO := NULL;
   END;

   V_COUNT := 0;      
   /*Se existe cardpio, verifica se existe retirada correspondente*/
   IF V_NRCARDAPIO IS NOT NULL THEN
       SELECT COUNT(1), MAX(NRLANCESTQ), MAX(NRRETIRADA)
           INTO V_COUNT, V_NRLANCTOEST, V_NRRETIRADA
       FROM RETIRADA RE
       WHERE RE.CDFILIAL = P_CDFILIAL
         AND RE.NRCARDAPIO = V_NRCARDAPIO
         AND RE.DTRETIRADA = P_DTCARDAPIO;
   END IF;
      
   /*Inclui cardpio e retirada caso no existam*/
   IF (V_COUNT = 0) THEN
      IF V_NRCARDAPIO IS NULL THEN
         NOVO_CODIGO('CARDAPIO'||P_CDFILIAL, V_NRCARDAPIO, 1);
         V_NRCARDAPIO := SUBSTR(V_NRCARDAPIO, 11, 10);
         INSERT INTO CARDADIA
            (CDFILIAL, NRCARDAPIO, CDSERVFILI, CDOPERADOR, DTCARDAPIO, DTMESCARD)
         VALUES
            (P_CDFILIAL, V_NRCARDAPIO, P_CDSERVICO, P_CDOPERADOR, P_DTCARDAPIO, ADD_MONTHS(LAST_DAY(P_DTCARDAPIO), -1) + 1);
      END IF;

      NOVO_CODIGO('LANCTOEST'||P_CDFILIPROC, V_NRLANCTOEST, 1);
      V_NRLANCTOEST := SUBSTR(V_NRLANCTOEST, 11, 10);
      
      INSERT INTO LANCTOESTOQ
        (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, IDALTCUSMED,
         NRLANCTONF, CDFILIMOVI, CDOPERADOR,  CDALMOXARIFE,  CDLOCALESTOQ)
      VALUES
        (P_CDFILIAL, V_NRLANCTOEST, '3', P_DTCARDAPIO, P_DSLANCAMENTO, 'N',
         NULL, P_CDFILIAL, P_CDOPERADOR,  ' ',  ' ');

      NOVO_CODIGO('RETIRADA'||P_CDFILIAL, V_NRRETIRADA, 1);
      V_NRRETIRADA := SUBSTR(V_NRRETIRADA, 11, 10);
      INSERT INTO RETIRADA
         (CDFILIAL, NRRETIRADA, NRCARDAPIO, DTRETIRADA, CDFILANCESTQ, NRLANCESTQ, IDRETIPRAT)
      VALUES
         (P_CDFILIAL, V_NRRETIRADA, V_NRCARDAPIO, P_DTRETIRADA, P_CDFILIPROC, V_NRLANCTOEST, 'N');
          
      V_NRSEQUITRETI := 0;

   ELSE
      SELECT MAX(NRSEQUITRETI)
         INTO V_NRSEQUITRETI
      FROM ITEMRETI
      WHERE CDFILIAL = P_CDFILIAL
        AND NRRETIRADA = V_NRRETIRADA
        AND NRCARDAPIO = V_NRCARDAPIO;
     
     IF V_NRSEQUITRETI IS NULL THEN
        V_NRSEQUITRETI := 0;
     END IF;
     
     V_NRSEQUITRETI := V_NRSEQUITRETI + 1;        
      
   END IF;

   FOR C_PRODRECEITA IN
      (SELECT RP.DTRETIRADALANC, RP.CDPRODUTO, RP.VRUNITRECEPRAT,
              SUM(RP.QTTOTARECEPRAT) AS QTTOTARECEPRAT, RP.IDRECEITAPRACA, PO.CDARVPROD,
              PO.NMPRODUTO, PO.SGUNIDADE, NVL(PO.VRFATOCONV, 1) AS VRFATOCONV,
              PO.CDPRODESTO
       FROM RECPRATPRODACAB RP, PRODUTO PO, PRODACABAPRATO PA 
       WHERE RP.CDFILIAL = P_CDFILIAL
         AND RP.DTRETIRADALANC = P_DTRETIRADA
         AND PO.CDPRODUTO = RP.CDPRODUTO
         AND PA.CDFILIAL = RP.CDFILIAL
         AND PA.DTRETIRADALANC = RP.DTRETIRADALANC
         AND PA.CDPRODACABADO = RP.CDPRODACABADO
         AND PA.IDSTATUSPRODACA = 'C'
       GROUP BY RP.DTRETIRADALANC, RP.CDPRODUTO, RP.VRUNITRECEPRAT,
                RP.IDRECEITAPRACA, PO.CDARVPROD, PO.NMPRODUTO, 
                PO.SGUNIDADE, NVL(PO.VRFATOCONV, 1),
                PO.CDPRODESTO  
      ) LOOP      

      CONSULTA_POSICAO(P_CDFILIAL,
                       C_PRODRECEITA.CDPRODUTO,
                       C_PRODRECEITA.DTRETIRADALANC-1,
                       ' ',
                       ' ',
                       ' ',
                       ' ',
                       'N',
                       V_QTDEESTO,
                       V_VALRESTO,
                       V_VALRBRUT);

      /*Calcula valor unitrio*/
      IF V_QTDEESTO > 0 THEN
         V_VALRBRUT := V_VALRBRUT / V_QTDEESTO;
      END IF;   

      IF (V_QTDEESTO < C_PRODRECEITA.QTTOTARECEPRAT) THEN
         V_QTCONSUMIDA := ROUND((C_PRODRECEITA.QTTOTARECEPRAT * C_PRODRECEITA.VRFATOCONV), 5);
      ELSE
         V_QTCONSUMIDA := ROUND((C_PRODRECEITA.QTTOTARECEPRAT * C_PRODRECEITA.VRFATOCONV), 5);
      END IF;

      IF V_QTCONSUMIDA > 0 THEN
         LOCALESTOQUE_PADRAO(P_CDFILIPROC, C_PRODRECEITA.CDPRODUTO, V_CDALMOXARIFE, V_CDLOCALESTOQ);
         IF V_CDALMOXARIFE IS NULL THEN
            V_CDALMOXARIFE := ' ';
         END IF;
         IF V_CDLOCALESTOQ IS NULL THEN
            V_CDLOCALESTOQ := ' ';
         END IF;
         
         CONSULTA_LOTE(V_IDUTILLOTE, P_CDFILIPROC, V_CDALMOXARIFE,
                       V_CDLOCALESTOQ, C_PRODRECEITA.CDPRODUTO,
                       V_LOTEESTQ, V_SUBLOTE);
         
         INSERT INTO ITLANCTOEST
            (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, QTLANCTOEST, VRUNILANCTO,
             VRTOTLANCTO, IDTIPOMOVI, DTLANCMOVI, IDORDEMCLC, QTTOTLANCTO,
             CDFILIMOVI, CDPRODMOVI, IDEVOLESTOQ, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, CDTIPORETI)
         VALUES
            (P_CDFILIPROC, V_NRLANCTOEST, LPAD(V_NRSEQUITRETI, 4, '0'), C_PRODRECEITA.CDPRODUTO, V_QTCONSUMIDA, V_VALRBRUT,
             C_PRODRECEITA.VRUNITRECEPRAT * V_QTCONSUMIDA, '3', P_DTCARDAPIO, '9', V_QTCONSUMIDA,
             P_CDFILIAL, C_PRODRECEITA.CDPRODESTO, 'S', V_CDALMOXARIFE, V_CDLOCALESTOQ, V_LOTEESTQ, V_SUBLOTE, V_CDTIPORETI);

         INSERT INTO ITEMRETI
            (CDFILIAL, NRRETIRADA, NRCARDAPIO, NRSEQUITRETI, CDPRODUTO, QTPREVISRETI, QTEXTRARETI, QTDEVOLVRETI, QTRETIRADA,
             CDFIITLANEST, NRLANCESTQ, NRSEQUITEM, CDTIPORETI)
         VALUES
            (P_CDFILIAL, V_NRRETIRADA, V_NRCARDAPIO, LPAD(V_NRSEQUITRETI, 4, '0'), C_PRODRECEITA.CDPRODUTO, V_QTCONSUMIDA, 0, 0, V_QTCONSUMIDA,
             P_CDFILIPROC, V_NRLANCTOEST, LPAD(V_NRSEQUITRETI, 4, '0'), V_CDTIPORETI);

      ELSE
         INSERT INTO ITEMRETI
            (CDFILIAL, NRRETIRADA, NRCARDAPIO, NRSEQUITRETI, CDPRODUTO, QTPREVISRETI, QTEXTRARETI, QTDEVOLVRETI, QTRETIRADA,
             CDFIITLANEST, NRLANCESTQ, NRSEQUITEM, CDTIPORETI)
         VALUES
            (P_CDFILIAL, V_NRRETIRADA, V_NRCARDAPIO, LPAD(V_NRSEQUITRETI, 4, '0'), C_PRODRECEITA.CDPRODUTO, V_QTCONSUMIDA, 0, 0, V_QTCONSUMIDA,
             NULL, NULL, NULL, V_CDTIPORETI);
      END IF;
      
      V_NRSEQUITRETI := V_NRSEQUITRETI + 1;

   END LOOP;

   UPDATE GERAPRODACABADO
   SET NRCARDAPIO = V_NRCARDAPIO,
       NRRETIRADA = V_NRRETIRADA
   WHERE CDFILIAL = P_CDFILIAL
     AND DTRETIRADALANC = P_DTRETIRADA;  

END;
/