import {
  Loader,
  Metadata,
  IDictionary,
  Http,
  IEventParam,
} from '@zeedhi/core';
  
import {
  Grid, Tabs, Text,
} from '@zeedhi/common';

export class Pla01305ConsultaNutrientesController {
  private utilities = Loader.getInstance('Utilities');
  
  public filterNutrientesGrid() {
    const filter: IDictionary = [];
    const gridConsultaNutrientes: Grid = Metadata.getInstance('CONSULTANUTRIENTES');

    this.utilities.onMountedFilterFind(filter, 'CDFILAUXPRAT', '=', 'P');
    this.utilities.onMountedFilterFind(filter, 'CDPRATO', '=', '0899040000');
    this.utilities.onMountedFilterFind(filter, 'CDSERVFILI', '=', null);
    this.utilities.onMountedFilterFind(filter, 'CDFILIAL', '=', null);
    this.utilities.onMountedFilterFind(filter, 'CDFILIPROD', '=', '0001');
    gridConsultaNutrientes.datasource.filter = filter;
    gridConsultaNutrientes.datasource.get();
  }
  
  public calcularNDPCAL() {
    const ndpcal: Text = Metadata.getInstance('NDPCALResult');
    const row = {
      CDFILIAL: null,
      CDFILIPROC: null,
      CDSERVFILI: null,
      CDFILAUXPRAT: 'P',
      CDPRATO: '0899040000',
      CDFILCENTPME: null,
      CDTIPOCARD: null,
      MERENDA: 'N',
    };
    
    Http.post('/calcularNDPCAL', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset.result.ndpcal[0];
      if (result) {
        const ndpcalNumber = parseInt(result, 10);
        ndpcal.text = `${ndpcalNumber.toFixed(2).replace('.', ',')}${'%'}`;
        this.filterNutrientesGrid();
      }
    });
  }
  
  public activateTab({ component }: IEventParam<Tabs>) {
    if (component.activeTab === 2) this.calcularNDPCAL();
  }
}
